/******************************************************************************/
/* cmfreport.h                                                   Rags Gardner */
/*                                                                            */
/* Rags Int., Inc.                                              December 1999 */
/* (c) RISC Application Growth Solutions International, Incorporated  1999    */
/*             (817) 267-2554       rags@compuserve.com                       */
/*                                                                            */
/******************************************************************************/

/*----------------------------------------------------------------------------*/
/*- defines                                                                  -*/
/*----------------------------------------------------------------------------*/
#define PRT_MAX  512                               /* maximum print line size */
#define FLD_MAX   30                   /* maximum report field width (+ null) */
#define LIST_SZ   60           /* maximum select/include/exclude list entries */
#define TOD_SZ    12                       /* report field width, time of day */
#define CLOCK_SZ  12                          /* report field width, os clock */
#define TIMER_SZ  12                       /* report field width, cics timers */
#define COUNT_SZ   8                          /* report field width, counters */
#define HWM_SZ     8           /* report field width, storage high water mark */

/*----------------------------------------------------------------------------*/
/*- Report Field Descriptions                                                -*/
/*- This is a static description of each potential field that might be       -*/
/*- selected for a report iteration                                          -*/
/*----------------------------------------------------------------------------*/
typedef struct FieldDesc_t
{
  cics_ulong_t empNum;                 /* cics cmf event monitor point number */
  int          fieldType;     /* type of cmf field (txt, timer, counter, ...) */
  enum       {   none=0,
                 Clk1T=1,                                          /* clock 1 */
                 Clk1Ct=2,                                 /* clock 1 counter */
                 Clk2T=3,                                          /* clock 2 */
                 Clk2Ct=4,                                 /* clock 2 counter */
                 Ctr1=5,                                         /* counter 1 */
                 Ctr2=6,                                         /* counter 2 */
                 Ctr3=7                                          /* counter 3 */
             } usrType;                            /* sub type, user emp only */
  int          outSize;                                  /* report field size */
  double       ValueCtr;                    /* accumulated count for this emp */
  double       ValueTime;            /* accumulated milliseconds for this emp */
  char         fieldName[FLD_MAX];   /* name used to select this report field */
  char         titleTxt[FLD_MAX];                        /* report title text */
};

/*----------------------------------------------------------------------------*/
/*- List of Selected Report Fields                                           -*/
/*- This is a dynamic list of report fields created from the command line    -*/
/*- field selections                                                         -*/
/*----------------------------------------------------------------------------*/
typedef struct FieldList_t
{
   struct FieldDesc_t *Fieldp;         /* pointer to the field report details */
};

/*----------------------------------------------------------------------------*/
/*- list of fields to check for included or excluded records                 -*/
/*----------------------------------------------------------------------------*/
typedef struct ExcIncList_t
{
   int                 field;                     /* the field to be compared */
   struct FieldDesc_t *Fieldp;         /* pointer to the field report details */
   enum                {                 /* greater than, less than, or equal */
                          EQUALS=0,
                          GREATERTHAN=1,
                          LESSTHAN=2
                       } oper;
   int                 value;                         /* value to be compared */
   char                text[FLD_MAX+2];     /* the text string to be compared */
};

/*----------------------------------------------------------------------------*/
/*- Summary Array Table Structures                                           -*/
/*----------------------------------------------------------------------------*/
struct SummaryTable_t
{
   struct SummaryRow_t *Head;                       /* first row in the table */
   struct FieldDesc_t  *Major;                   /* the major column selected */
   struct FieldDesc_t  *Minor;                   /* the minor column selected */
};

struct SummaryRow_t
{
   struct SummaryRow_t  *Next;                               /* chain pointer */
   char                  MajorValue[12];            /* major field data value */
   char                  MinorValue[12];            /* minor field data value */
   double                ItemCount;        /* records accumulated in this row */
   struct SummaryItem_t *ItemS;               /* the item detail accumulators */
};

struct SummaryItem_t
{
   struct FieldDesc_t *Fieldp;         /* pointer to the field report details */
   double RowCount;                       /* accumulated counter for this row */
   double RowTime;                          /* accumulated timer for this row */
};

/*----------------------------------------------------------------------------*/
/*- function prototypes                                                      -*/
/*----------------------------------------------------------------------------*/
int                 cmfopts    (int argc, char *argv[],
                                char                          MonDataFile[],
                                char                          TopTitle[],
                                struct  FieldList_t           FieldList[],
                                struct  ExcIncList_t          IncludeList[],
                                struct  ExcIncList_t          ExcludeList[]);
int                 MonTitle   (CICS_EMP_TDQ_RecordDetails_t *RecordDetails,
                                struct FieldDesc_t           *Fieldp,
                                char                          titleTxt[]);
int                 MonCompare (CICS_EMP_TDQ_RecordDetails_t *RecordDetails,
                                struct ExcIncList_t           CompareItem);
int                 MonExpand  (CICS_EMP_TDQ_RecordDetails_t *RecordDetails,
                                struct FieldDesc_t           *Fieldp,
                                char                          fieldValue[],
                                struct SummaryRow_t          *row);

int                 Field2Num  (char *textStr);
int                 ValidateIE (struct ExcIncList_t          *ExcIncItem,
                                char                         *argemp,
                                char                         *argvalue);
struct FieldDesc_t *emp2Fieldp (cics_ulong_t EMP, char *textStr);
int                 insertRow  (struct SummaryRow_t          *row);
